#!/bin/csh
#
# --- script to submit Hycom simulations modified by
# --- 016nqs.com from ATLb4.00 directory
# --- Ver 01, Gatimu 11/11/05
# 
# --- this script is the one submitted with the msub command
# --- it reads the LIST file and it creates the specific 
# --- SCRIPT with using the file *.awk
#
#
# --- check that the C comment command is available.
#
set echo
C
C --- Preamble
C
C >& /dev/null
if (! $status) then
  if (-e ${home}/hycom/ALL/bin/C) then
    set path = ( ${path} ${home}/hycom/ALL/bin )
  else
    echo "Please put the command hycom/ALL/bin/C in your path"
  endif
endif
#
set echo
C
if ($?JOBNAME) then
    setenv QSUB_REQNAME R${JOBNAME}
    setenv QSUB_REQID   $$
endif
C
C --- Automatic Run Script.
C --- Submit with msub, or msub_csh, or msub_nqs command.
C --- Multiple segment version, set no. of segments on foreach below.
C
C --- R is region name.
C --- V is source code version number.
C --- T is topography number.
C --- K is number of layers.
C --- E is expt number.
C --- P is primary path.
C --- D is permanent directory.
C --- S is scratch directory, must not be the permanent directory.
C
setenv R FWFb4.00
setenv V 2.1.34
setenv T 01
setenv K 01
setenv E 011
setenv P hycom/${R}/expt_01.1
setenv D ~/$P/data
setenv S $D/SCRATCH
C
C --- move in the expt dir
C
cd     ~/$P
C
/bin/ls -laFq
C
C --- check the RUNNING flag.
C
if ( -e RUNNING && ! -e RUNNING_$QSUB_REQID) then
C
C --- MODEL IS ALREADY RUNNING - EXIT.
C
  exit
endif
touch RUNNING
touch RUNNING_$QSUB_REQID
C
C --- Number of segments specified by '( SEG1 SEG2 ... )' in foreach.
C --- Use '( SEG1 )' for a single segment per run.
C
#foreach seg (SEG1 SEG2 SEG3 SEG4 SEG5)
foreach seg (SEG1)
echo "Starting SEGMENT " $seg
C
C --- Check the model list.
C
touch LIST
if ( -z LIST ) then
C
C --- MODEL RUN IS COMPLETE - EXIT.
C
  /bin/rm -f RUNNING
  /bin/rm -f RUNNING_$QSUB_REQID
  exit
endif
C
C --- Take the next run from LIST.
C
setenv Y01 `head -1 LIST | awk '{print $1}'`
C
if ( `echo $Y01 | egrep 'com' | wc -l` != 0 ) then
C
C --- Precalculated model script.
C
  setenv SCRIPT ${Y01}
  if (! -e $SCRIPT ) then
C
C --- PREDEFINED SCRIPT DOES NOT EXIST - EXIT.
C
    /bin/ls -laFq
    /bin/rm -f RUNNING
    /bin/rm -f RUNNING_$QSUB_REQID
    exit
  endif
else
C
C --- Generate the next model script.
C
  setenv AB `head -1 LIST | awk '{print $2}'`
  setenv SCRIPT ${E}y${Y01}${AB}.com
  /bin/rm -f ${SCRIPT}
  awk -f ${E}.awk y01=${Y01} ab=${AB} ${E}.com > ${SCRIPT}
endif
C
C --- create SCRATCH dir and its whole path
C
mkdir -p $S
C
C --- Run the Script.
C
set script = $SCRIPT
set reqname = `echo ${QSUB_REQNAME} | awk '{print substr($0,2)}'`
/bin/ln -fs ${reqname}.log $script:r.log
C
mkdir -p $S
cp  ${SCRIPT} $S/${SCRIPT}
cd  $S
C
C ---------------------------------------------------------------------------
/bin/csh ${SCRIPT}
C ---------------------------------------------------------------------------
C
C
C --- move again back in the expt dir
C
cd ~/$P
C
C --- Clean Up.
C
/bin/ls -laFq
C
/bin/mv LIST LISTOLD
head -1 LISTOLD | comm -23 LISTOLD - > LIST
diff LIST LISTOLD
/bin/rm -f LISTOLD
C
C --- end of SEGMENT foreach loop
C
end
C
C --- Final Clean Up.
C
/bin/rm -f RUNNING
/bin/rm -f RUNNING_$QSUB_REQID
C
touch LIST
if ( -z LIST ) then
C
C --- MODEL RUN IS COMPLETE - EXIT.
C
  exit
endif
C
C --- Submit the next job.
C
cat - > $S/$$.awk <<'E-o-A'
BEGIN { for ( i=65;i <= 89; i++)
		c[sprintf("%c",i)] = sprintf("%c",i+1)
}
/[0-9]$/  { printf("%s%s\n",$0,"A")}
/[A-Y]$/  { printf("%s%s\n",substr($0,1,length($0)-1),c[substr($0,length($0))])}
/[1-9]Z$/ { printf("%s%s%s\n",substr($0,1,length($0)-2),substr($0,length($0)-1,1)+1,"A")}
/0Z$/     { next }
'E-o-A'
#
set newname = `echo ${reqname} | awk -f $S/$$.awk`
#
/bin/rm -f $S/$$.awk
if ( `echo $newname | wc -l` != 0 ) then
    if ($?JOBNAME) then
        setenv JOBNAME ${newname}
#    fix a Linux NFS/tcsh bug
        (/bin/csh ${E}nqs.com |& cat - >& ${newname}.log) &
    else
        qsub -mb -me -p 63 -r ${newname} \
                       -eo -o ${newname}.log ${E}nqs.com
    endif
else
C
C --- newname failed
C
    echo $newname
    echo $newname | wc -l
endif
C
C --- Exit.
C
exit
